package com.timtech.profile.service.impl;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;

import com.timtech.profile.dto.ProfileDto;
import com.timtech.profile.entity.Profile;
import com.timtech.profile.exception.BusinessException;
import com.timtech.profile.repository.ProfileRepository;
import com.timtech.profile.service.ProfileService;
import com.timtech.profile.util.Utils;

import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
@Slf4j
public class ProfileServiceImpl implements ProfileService {

	@Autowired
	private ProfileRepository profileRepository;

	public Mono<ProfileDto> save(ProfileDto profileDto) {
		log.debug("Request to save Profile " + profileDto);
		if (profileDto == null) {
			return Mono.error(new RuntimeException("Profile Should Not be NULL"));
		}

		if (Utils.builder().build().isValidDataFormats(profileDto)) {
			return profileRepository.existsByPermanentAccountNumber(profileDto.getPermanentAccountNumber())
					.zipWith(profileRepository.existsByAdharNumber(profileDto.getAdharNumber())).flatMap(tuple -> {
						if (tuple.getT1() || tuple.getT2()) {
							return Mono.error(
									new ResponseStatusException(HttpStatus.CONFLICT, "Duplicate PAN or Aadhar Number"));
						} else {
							Profile profileSave = new Profile();
							profileDto.setStatus("ACTIVE");
							BeanUtils.copyProperties(profileDto, profileSave);
							return profileRepository.save(profileSave).map(this::convertProfileToDto);
						}
					})
					.onErrorResume(DuplicateKeyException.class,
							ex -> Mono.error(
									new ResponseStatusException(HttpStatus.CONFLICT, "Duplicate PAN or Aadhar Number")))
					.onErrorResume(ex -> Mono.error(
							new BusinessException(HttpStatus.INTERNAL_SERVER_ERROR, "Internal Server Error !!!")));
		} else {
			return Mono.error(new BusinessException(HttpStatus.BAD_REQUEST,
					"Invalid PAN or Aadhar Number or First Name or Last Name or Date of Birth or Mobile Number"));
		}
	}

	public Mono<ProfileDto> update(ProfileDto profileDto) {
		log.debug("Request to update profile " + profileDto);

		if (profileDto == null) {
			return Mono.error(new RuntimeException("Profile Should Not be NULL"));
		}

		if (Utils.builder().build().isValidDataFormats(profileDto)) {

			return findOne(profileDto.getProfileId()).flatMap(existingProfileDto -> {
				if ("ACTIVE".equals(existingProfileDto.getStatus())) {
					Profile profile = convertDtoToProfile(existingProfileDto);
					profile.setStatus("IN-ACTIVE");
					return profileRepository.save(profile).map(this::convertProfileToDto)
							.onErrorMap(DuplicateKeyException.class, ex -> {
								throw new RuntimeException("Duplicate PAN or Aadhar Number");
							});
				} else {
					return Mono.error(new RuntimeException("Profile has been Deleted !!!"));
				}

			});
		} else {
			return Mono.error(new BusinessException(HttpStatus.BAD_REQUEST,
					"Invalid PAN or Aadhar Number or First Name or Last Name or Date of Birth or Mobile Number"));
		}

	}

	public Flux<ProfileDto> findAll() {
		log.debug("Request to find all profiles ");
		return profileRepository.findByStatus("ACTIVE").map(this::convertProfileToDto);
	}

	public Mono<ProfileDto> findOne(String profileId) {
		log.debug("Request to find one profile ");
		return profileRepository.findById(profileId).map(this::convertProfileToDto);
	}

	public Mono<String> deleteOne(String profileId) {
		log.debug("Request to delete one profile ");

		return findOne(profileId).flatMap(profileDto -> {
			Profile profile = convertDtoToProfile(profileDto);
			profile.setStatus("IN-ACTIVE");
			return profileRepository.save(profile);
		}).flatMap(savedProfile -> Mono.just("SUCCESS")).onErrorResume(throwable -> Mono.just("FAILURE"));
	}

	private ProfileDto convertProfileToDto(Profile profile) {
		ProfileDto profileDto = ProfileDto.builder().build();
		BeanUtils.copyProperties(profile, profileDto);
		return profileDto;
	}

	private Profile convertDtoToProfile(ProfileDto profileDto) {
		Profile profile = Profile.builder().build();
		BeanUtils.copyProperties(profileDto, profile);
		return profile;
	}

}
